/* 
   $Id: callbacks.c,v 1.14 2001/12/21 02:55:24 shibajun Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgse.

   sgse is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgse is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgse; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define YES 1
#define NO 0

extern GtkWidget *mainapp;
static char *filename = NULL;
static gboolean CHANGED = FALSE ;
#define MAXPATHLEN 1024
static char cwd[MAXPATHLEN] = "" ;

/*----------------------------------------------------------*/
static gboolean compare_buff_file(void)
{
  FILE *fp ;
  gchar *buff ; /* for buff */
  char buf[1024] ; /* for file */
  GtkWidget *textArea ;
  int nbuff = 0 ; /* buff length */
  int nbuf = 0 ; /* file length */

  if ((fp = fopen (filename, "r")) == NULL) {
    return FALSE;
  }

  /* lookup  widget */
  textArea = lookup_widget(mainapp,"text1"); /* search widget */
  
  /* get buffer length */
  buff = gtk_editable_get_chars (GTK_EDITABLE(textArea), 0, -1) ;
  nbuff = strlen(buff) ;

  /* get file length */
  while (!feof(fp)) {
    nbuf+=nbuf ;
    nbuf = fread (buf, 1, 1024, fp) ;
  }

#if 0
  printf ("nbuff,nbuf = %d,%d\n", nbuff, nbuf) ;
#endif

  /* compare size */
  if (nbuff == nbuf) {
    return FALSE ;
  }
  return TRUE ;
}

/*----------------------------------------------------------*/
static void cleanText(void)
{
    GtkWidget *textArea;
    
    textArea = lookup_widget(mainapp,"text1"); /* search widget */
    gtk_editable_delete_text(GTK_EDITABLE(textArea),0,-1); /* delete entire */

    if (filename != NULL) {
      filename = NULL;
      g_free(filename);
    }
}

/*----------------------------------------------------------*/
void read_file(char *fn)
{
    FILE *fp;
    char buf[1024];
    GtkWidget *textArea;
    int pos;
    int n;

    if ((fp = fopen(fn,"r")) == NULL) {
      GtkWidget *msg;

      msg = gnome_error_dialog(_("Can't open file\n"));
      gnome_dialog_run_and_close(GNOME_DIALOG(msg));
      return;
    }

    cleanText();
    textArea = lookup_widget(mainapp,"text1");
    pos = 0;
    while ((n = fread(buf,1,1024,fp)) > 0) {
      gtk_editable_insert_text(GTK_EDITABLE(textArea),buf,n,&pos);
    }
    fclose(fp);
    filename = g_strdup(fn);
    CHANGED = FALSE ;
}

/*----------------------------------------------------------*/
static char *getFileName(void)
{
  int bid; /* button ID */
  GtkWidget *dialog;
  GtkWidget *fe;
  GtkWidget *combo_entry1 ;
  char *fn = NULL;
  char *tmp = NULL ;

  dialog = create_filesel_dialog();
  gtk_widget_set_usize (GTK_WIDGET(dialog), 230, -2) ;
  fe = lookup_widget(dialog,"fileentry1");
  gtk_widget_set_usize (GTK_WIDGET(dialog), 230, -2) ;
  combo_entry1 = lookup_widget (dialog, "combo_entry1") ;
  if (strcmp (cwd, "") == 0) {
    getcwd (cwd, MAXPATHLEN) ;
    strcat (cwd, "/") ; /* should be terminated with back-slash */
  }
#if 0
  g_print ("get cwd = %s\n", cwd) ;
#endif
  gtk_entry_set_text (GTK_ENTRY(combo_entry1), cwd) ;
  bid = gnome_dialog_run(GNOME_DIALOG(dialog));
  if (bid == 0) { /* OK button */
    fn = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(fe),FALSE);
    tmp = g_dirname (fn) ;
  }
  if (tmp != NULL) {
    sprintf (cwd, "%s", tmp) ;
    strcat (cwd, "/") ; /* should be terminated with back-slash */
  }
#if 0
  g_print ("fn = %s\n", fn) ;
  g_print ("getdir = %s\n", cwd) ;
#endif
  gnome_dialog_close(GNOME_DIALOG(dialog));
  return fn;
}

/*----------------------------------------------------------*/
static void display_about (void)
{
  GtkWidget *about ;

  about = create_about_dialog () ;
  gtk_widget_show (about) ; 
                            
}

/*----------------------------------------------------------*/
static void save_file(void)
{
    FILE *fp;
    GtkWidget *textArea;
    char *p;

    if (filename == NULL) { /* filename exist? */
      filename = getFileName();
      if (filename == NULL) return;
    }
    if ((fp = fopen(filename,"w")) == NULL) {
      GtkWidget *msg;

      msg = gnome_error_dialog(_("Can't open file\n"));
      gnome_dialog_run_and_close(GNOME_DIALOG(msg));
      filename = NULL ;
      return;
    }

    textArea = lookup_widget(mainapp,"text1");
    p = gtk_editable_get_chars(GTK_EDITABLE(textArea),0,-1);
    if (p != NULL) {
      fwrite(p,1,strlen(p),fp);
      g_free(p);
    }
    fclose(fp);
    CHANGED = FALSE ;
}


/*----------------------------------------------------------*/
static void do_confirm ()
{
  GtkWidget *confirm ;
  int bid ; /* button ID */

  confirm = create_confirm_dialog () ;
  bid = gnome_dialog_run (GNOME_DIALOG(confirm)) ;

  if (bid !=0) { /* not OK button, just return */
#ifdef AHO
    printf ("do_confirm:cancel\n") ;
#endif
    gnome_dialog_close (GNOME_DIALOG(confirm)) ;
    return ;
  }    
  gnome_dialog_close (GNOME_DIALOG(confirm)) ;
  save_file () ;
}

/*----------------------------------------------------------*/
void
on_new_file1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (CHANGED) { /* already save it? */
#ifdef AHO
    printf ("filename = %s\n", filename) ;
#endif
    do_confirm () ;
  }
  cleanText () ;
  CHANGED = FALSE ;
}

/*----------------------------------------------------------*/
void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  char *fn;

  if (CHANGED) { /* already save it? */
    do_confirm () ;
  }

  fn = getFileName();
  if (fn != NULL) {
    read_file(fn);
    g_free(fn);
    CHANGED = FALSE ;
  }
}


/*----------------------------------------------------------*/
void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  save_file();
}


/*----------------------------------------------------------*/
void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  char *fn;

  fn = getFileName();
  if (fn != NULL) {
    if (filename != "") {
      g_free(filename);
    }
    filename = fn;
    save_file();
  }

}


/*----------------------------------------------------------*/
void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (CHANGED) { /* already save it? */
#ifdef AHO
    printf ("filename = %s\n", filename) ;
#endif
    do_confirm () ;
  }
  gtk_main_quit () ;
}


/*----------------------------------------------------------*/
void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *textArea;

    textArea = lookup_widget(mainapp,"text1"); /* search widget */
    gtk_editable_cut_clipboard (GTK_EDITABLE(textArea)) ;
}


/*----------------------------------------------------------*/
void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *textArea;

    textArea = lookup_widget(mainapp,"text1"); /* search widget */
    gtk_editable_copy_clipboard (GTK_EDITABLE(textArea)) ;

}


/*----------------------------------------------------------*/
void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *textArea;

    textArea = lookup_widget(mainapp,"text1"); /* search widget */
    gtk_editable_paste_clipboard (GTK_EDITABLE(textArea)) ;

}


/*----------------------------------------------------------*/
void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  cleanText () ;
}

/*----------------------------------------------------------*/
void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  /*
  GtkWidget *about ;
  gchar *authors[] = {NULL} ;

  //  gnome_ok_dialog ("This is sgse") ;
  about = gnome_about_new (_("sgse"), VERSION,
			   "(C) 2001 AXE,Inc.",
			   authors,
			   _("I hope this application makes you happy"),
			   NULL) ;
  gtk_widget_show (about) ;
  */
  display_about () ;
}

/*----------------------------------------------------------*/
void
on_button4_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_main_quit () ;
}

void
new_button_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  on_new_file1_activate (0, user_data) ;
}


void
open_button_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  on_open1_activate (0, user_data) ;
}


void
save_button_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  on_save1_activate (0, user_data) ;
}

void
on_text1_changed                       (GtkEditable     *editable,
                                        gpointer         user_data)
{
  CHANGED = TRUE ;
}

void
cut_button_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  on_cut1_activate (0, user_data) ;
}


void
copy_button_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  on_copy1_activate (0, user_data) ;
}


void
paste_button_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  on_paste1_activate (0, user_data) ;
}



